/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.powershell.model;

import java.util.ArrayList;
import java.util.List;

import com.redhat.rhevm.api.common.util.StatusUtils;
import com.redhat.rhevm.api.model.Cluster;
import com.redhat.rhevm.api.model.Host;
import com.redhat.rhevm.api.model.HostStatus;
import com.redhat.rhevm.api.model.PowerManagement;
import com.redhat.rhevm.api.model.Option;
import com.redhat.rhevm.api.model.Options;
import com.redhat.rhevm.api.model.Statistic;
import com.redhat.rhevm.api.model.Statistics;
import com.redhat.rhevm.api.powershell.enums.PowerShellVdsSpmStatus;
import com.redhat.rhevm.api.powershell.enums.PowerShellVdsType;
import com.redhat.rhevm.api.powershell.util.PowerShellParser;

import static com.redhat.rhevm.api.powershell.util.PowerShellUtils.last;

public class PowerShellHost {

    private static final String HOST_TYPE = "RhevmCmd.CLIHost";

    private static HostStatus parseStatus(String s) {
        if (s.equals("Down"))                      return HostStatus.DOWN;
        if (s.equals("Error"))                     return HostStatus.ERROR;
        if (s.equals("Initializing"))              return HostStatus.INITIALIZING;
        if (s.equals("Installing"))                return HostStatus.INSTALLING;
        if (s.equals("Install Failed"))            return HostStatus.INSTALL_FAILED;
        if (s.equals("Maintenance"))               return HostStatus.MAINTENANCE;
        if (s.equals("Non Operational"))           return HostStatus.NON_OPERATIONAL;
        if (s.equals("Non Responsive"))            return HostStatus.NON_RESPONSIVE;
        if (s.equals("Pending Approval"))          return HostStatus.PENDING_APPROVAL;
        if (s.equals("Preparing For Maintenance")) return HostStatus.PREPARING_FOR_MAINTENANCE;
        if (s.equals("Non-Responsive"))            return HostStatus.CONNECTING;
        if (s.equals("Reboot"))                    return HostStatus.REBOOT;
        if (s.equals("Unassigned"))                return HostStatus.UNASSIGNED;
        if (s.equals("Up"))                        return HostStatus.UP;
        else assert false : s;
        return null;
    }

    private static Options parsePowerManagementOptions(String options) {
        if (options == null) {
            return null;
         }

        Options ret = new Options();

        String[] opts = options.split(",");
        for (int i = 0; i < opts.length; i++) {
            String[] parts = opts[i].split("=");

            Option option = new Option();
            option.setName(parts[0]);
            option.setValue(parts[1]);
            ret.getOptions().add(option);
        }

        return ret;
    }

    private static PowerManagement parsePowerManagement(PowerShellParser.PowerManagement parsed) {
        PowerManagement ret = new PowerManagement();
        ret.setType(parsed.getType());
        ret.setEnabled(parsed.getEnabled());
        ret.setAddress(parsed.getAddress());
        ret.setUsername(parsed.getUsername());
        ret.setOptions(parsePowerManagementOptions(parsed.getOptions()));
        return ret;
    }

    public static List<Host> parse(PowerShellParser parser, String output) {
        List<Host> ret = new ArrayList<Host>();

        for (PowerShellParser.Entity entity : parser.parse(output)) {
            if (HOST_TYPE.equals(entity.getType())) {
                ret.add(parseHost(entity));
            } else if (PowerShellHostStatisticsParser.isMemory(entity)) {
                getStatistics(ret).addAll(PowerShellHostStatisticsParser.parseMemoryStats(entity));
            } else if (PowerShellHostStatisticsParser.isCpu(entity)) {
                getStatistics(ret).addAll(PowerShellHostStatisticsParser.parseCpuStats(entity));
            } else if (PowerShellHostStatisticsParser.isKsm(entity)) {
                getStatistics(ret).addAll(PowerShellHostStatisticsParser.parseKsmStats(entity));
            }
        }

        return ret;
    }

    private static Host parseHost(PowerShellParser.Entity entity) {
        Host host = new Host();
        host.setId(entity.get("hostid", String.class, Integer.class).toString());
        host.setCluster(new Cluster());
        host.getCluster().setId(entity.get("hostclusterid", String.class, Integer.class).toString());
        host.setAddress(entity.get("address"));
        host.setPort(entity.get("port", Integer.class));
        host.setName(entity.get("name"));
        host.setStatus(StatusUtils.create(parseStatus(entity.get("status"))));
        host.setType(PowerShellVdsType.valueOf(entity.get("hosttype")).map());
        host.setStorageManager(entity.get("spmstatus", PowerShellVdsSpmStatus.class).map());
        host.setPowerManagement(parsePowerManagement(entity.get("powermanagement",
                                                                PowerShellParser.PowerManagement.class)));
        return host;
    }

    private static List<Statistic> getStatistics(List<Host> hosts) {
        Host host = last(hosts);
        if (!host.isSetStatistics()) {
            host.setStatistics(new Statistics());
        }
        return host.getStatistics().getStatistics();
    }
}
