/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.powershell.resource;

import java.text.MessageFormat;
import java.util.List;

import javax.ws.rs.Produces;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriBuilder;

import com.redhat.rhevm.api.resource.MediaType;

import com.redhat.rhevm.api.model.BaseResource;
import com.redhat.rhevm.api.model.Tag;
import com.redhat.rhevm.api.model.Tags;
import com.redhat.rhevm.api.common.resource.UriInfoProvider;
import com.redhat.rhevm.api.common.util.LinkHelper;
import com.redhat.rhevm.api.common.util.ReflectionHelper;
import com.redhat.rhevm.api.powershell.model.PowerShellTag;
import com.redhat.rhevm.api.powershell.util.PowerShellCmd;
import com.redhat.rhevm.api.powershell.util.PowerShellParser;
import com.redhat.rhevm.api.powershell.util.PowerShellPoolMap;
import com.redhat.rhevm.api.powershell.util.PowerShellUtils;
import com.redhat.rhevm.api.resource.AssignedTagResource;
import com.redhat.rhevm.api.resource.AssignedTagsResource;

import static com.redhat.rhevm.api.common.util.CompletenessAssertor.validateParameters;

@Produces({MediaType.APPLICATION_XML, MediaType.APPLICATION_JSON, MediaType.APPLICATION_X_YAML})
public class PowerShellAssignedTagsResource
    extends UriProviderWrapper
    implements AssignedTagsResource {

    protected Class<? extends BaseResource> parentType;
    protected String parentId;

    public PowerShellAssignedTagsResource(Class<? extends BaseResource> parentType,
                                          String parentId,
                                          PowerShellPoolMap shellPools,
                                          PowerShellParser parser,
                                          UriInfoProvider uriProvider) {
        super(null, shellPools, parser, uriProvider);
        this.parentType = parentType;
        this.parentId = parentId;
    }

    public Class<? extends BaseResource> getParentType() {
        return parentType;
    }

    public String getParentId() {
        return parentId;
    }

    public List<Tag> runAndParse(String command) {
        return PowerShellTag.parse(getParser(), PowerShellCmd.runCommand(getPool(), command));
    }

    public Tag runAndParseSingle(String command) {
        List<Tag> tags = runAndParse(command);
        return !tags.isEmpty() ? tags.get(0) : null;
    }

    public Tag addLinks(Tag tag) {
        ReflectionHelper.assignChildModel(tag, parentType).setId(parentId);

        return LinkHelper.addLinks(getUriInfo(), tag);
    }

    private String getParentIdArg() {
        return MessageFormat.format("-{0}id {1}",
                                    parentType.getSimpleName().toLowerCase(),
                                    PowerShellUtils.escape(parentId));
    }

    public Tag getTag(String id) {
        return runAndParseSingle("get-tag " + PowerShellUtils.escape(id));
    }

    @Override
    public Tags list() {
        Tags ret = new Tags();
        for (Tag tag : runAndParse("get-tags " + getParentIdArg())) {
            ret.getTags().add(addLinks(tag));
        }
        return ret;
    }

    @Override
    public Response add(Tag tag) {
        validateParameters(tag, "id|name");

        StringBuilder buf = new StringBuilder();

        if (tag.isSetId()) {
            buf.append("$tag = get-tag " + PowerShellUtils.escape(tag.getId()) + "; ");
        } else {
            buf.append("$tag = get-tags | ? { $_.name -eq " + PowerShellUtils.escape(tag.getName()) + " }; ");
        }

        buf.append("attach-tag -tagobject $tag " + getParentIdArg());

        tag = addLinks(runAndParseSingle(buf.toString()));

        UriBuilder uriBuilder = getUriInfo().getAbsolutePathBuilder().path(tag.getId());

        return Response.created(uriBuilder.build()).entity(tag).build();
    }

    @Override
    public void remove(String id) {
        StringBuilder buf = new StringBuilder();

        buf.append("$tag = get-tag " + PowerShellUtils.escape(id) + "; ");
        buf.append("detach-tag -tagobject $tag " + getParentIdArg());

        PowerShellCmd.runCommand(getPool(), buf.toString());
    }

    @Override
    public AssignedTagResource getAssignedTagSubResource(String id) {
        return new PowerShellAssignedTagResource(id, this);
    }
}
